module {{cookiecutter.module_model_namespace}}.{{cookiecutter.program_name}} exposing (..)

{-| Do not manually edit this file, it was auto-generated by djelm
<https://github.com/Confidenceman02/django-elm>
-}

import Json.Decode as Decode
import Json.Decode.Pipeline exposing (required, optional, hardcoded)


type alias ToModel =
    {{cookiecutter.alias_type}}


{{cookiecutter.decoder_body}}
